set.seed(123) # Ensure reproducibility

lepage.test <- function(x, y = NA, g = NA, method = NA, n.mc = 10000){
  
  ##Adapted from kruskal.test()##
  if (is.list(x)) {
    if (length(x) < 2L) 
      stop("'x' must be a list with at least 2 elements")
    y <- x[[2]]
    x <- x[[1]]
  }
  else {
    if(min(is.na(y)) != 0){
      k <- length(unique(g))
      if (length(x) != length(g)) 
        stop("'x' and 'g' must have the same length")
      if (k < 2) 
        stop("all observations are in the same group")
      y <- x[g == 2]
      x <- x[g == 1]
    }
  }
  #####################
  
  outp <- list()
  outp$m <- m <- length(x)
  outp$n <- n <- length(y)
  outp$n.mc <- n.mc
  N <- outp$m + outp$n
  outp$ties <- (length(c(x, y)) != length(unique(c(x, y))))
  even <- (outp$m + outp$n + 1)%%2
  outp$stat.name <- "Lepage D"
  
  
  ##When the user doesn't give us any indication of which method to use, try to pick one.
  if(is.na(method)){
    if(choose(outp$m + outp$n, outp$n) <= 10000){
      method <- "Exact"
    }
    if(choose(outp$m + outp$n, outp$n) > 10000){
      method <- "Monte Carlo"
    }
  }
  #####################################################################
  outp$method <- method
  
  tmp.W <- rank(c(x, y))
  
  our.data <- rbind(c(x, y), c(rep(1, length(x)), rep(0, length(y))))
  sorted <- our.data[1, order(our.data[1, ]) ]
  x.labels <-our.data[2, order(our.data[1, ]) ]
  
  med <- ceiling(N / 2)
  if(even){no.ties <- c(1:med, med:1)}
  if(!even){no.ties <- c(1:med, (med - 1):1)}
  
  obs.group <- numeric(N)
  group.num <- 1
  for(i in 1:N){
    if(obs.group[i] == 0){
      obs.group[i] <- group.num
      for(j in i:N){
        if(sorted[i] == sorted[j]){
          obs.group[j] <- obs.group[i]
        }
      }
      group.num <- group.num + 1;
    }
  }
  
  group.ranks <- tapply(no.ties, obs.group, mean)
  
  tied.ranks <- numeric(N)
  for(i in 1:group.num){
    tied.ranks[which(obs.group == as.numeric(names(group.ranks)[i]))] <- group.ranks[i]
  }
  
  tmp.C <- c(tied.ranks[x.labels == 1], tied.ranks[x.labels == 0])
  
  ##Only needs to depend on y values
  D.calc <- function(C.vals, W.vals){
    
    if(even){
      exp.C <- n * (N + 2) / 4
      var.C <- m * n * (N + 2) * (N - 2) / (48 * (N - 1))
    }
    if(!even){
      exp.C <- n * (N + 1)^2 / (4 * N)
      var.C <- m * n * (N + 1) * (3 + N^2) / (48 * N^2)
    }
    W.obs <- sum(W.vals)
    W.star <- (W.obs - n * (N + 1) / 2) / sqrt(m * n * (N + 1) / 12)
    C.star <- (sum(C.vals) - exp.C) / sqrt(var.C)
    return(W.star^2 + C.star^2)
  }
  
  outp$obs.stat <- D.calc(tmp.C[(m + 1):N], tmp.W[(m + 1):N])
  
  if(outp$method == "Exact"){
    possible.orders <- gtools::combinations(outp$m + outp$n, outp$n)
    
    possible.C <- t(apply(possible.orders, 1, function(x) tmp.C[x]))
    possible.W <- t(apply(possible.orders, 1, function(x) tmp.W[x]))
    
    theor.dist <- numeric(nrow(possible.C))
    for(i in 1:nrow(possible.C)){
      theor.dist[i] <- D.calc(possible.C[i, ], possible.W[i, ])
    }
    
    outp$p.value <- mean(theor.dist >= outp$obs.stat)
  }
  
  if(outp$method == "Asymptotic"){
    outp$p.value <- (1 - pchisq(outp$obs.stat, 2))
  }
  
  if(outp$method == "Monte Carlo"){
    outp$p.value <- 0
    for(i in 1:n.mc){
      mc.sample <- sample(1:N, n)
      
      if(D.calc(tmp.C[mc.sample], tmp.W[mc.sample]) >= outp$obs.stat){
        outp$p.value = outp$p.value + 1 / n.mc
      }
    }
  }
  
  cat("\nLepage two-sample location-scale test\n")
  cat("\nNull hypothesis: The locations and scales of the two population distributions are equal.\n")
  cat("Alternative hypothesis: The locations and/or scales of the two population distributions differ.\n")
  cat(paste("\nD = ", round(outp$obs.stat, 3), ", p-value = ", round(outp$p.value, 4), "\n\n", sep=""))
  
  #class(outp)="NSM3Ch5p"
  return(outp)
}
lapage <- function(mu_x, se_mu_x, sigma_x, se_sigma_x, n_x, 
                   mu_y, se_mu_y, sigma_y, se_sigma_y, n_y, 
                   model_x, model_y, n_sim) {  
  
  values <- data.frame(p = double(n_sim), D = double(n_sim))
  
  for (i in 1:n_sim) {
    # Sample means and standard deviations from their respective distributions
    mean_x <- rnorm(1, mean = mu_x, sd = se_mu_x)
    mean_y <- rnorm(1, mean = mu_y, sd = se_mu_y)
    
    sd_x <- rnorm(1, mean = sigma_x, sd = se_sigma_x)
    sd_y <- rnorm(1, mean = sigma_y, sd = se_sigma_y)
    
    # Ensure standard deviations are positive
    sd_x <- max(sd_x, 0.01)
    sd_y <- max(sd_y, 0.01)
    
    # Generate data
    x <- rnorm(n_x, mean = mean_x, sd = sd_x)
    y <- rnorm(n_y, mean = mean_y, sd = sd_y)
    
    # Perform Lepage test
    test <- lepage.test(x, y)
    values$p[i] <- test$p.value
    values$D[i] <- test$obs.stat
  }
  
  # Calculate the proportion of p-values < 0.05
  prop_reject <- mean(values$p < 0.05)
  
  # Print the result
  cat(paste0(model_x, " vs ", model_y, "\n"))
  cat(paste0("Reject in ", prop_reject * 100, "%\n"))
  
  return(values)
}



# Load models
MXL_wtp_Control_Not_Pred_model <- apollo_loadModel("Estimation_results/mxl/prediction/Split_samples/MXL_wtp_Control_Not_Pred")
MXL_wtp_Control_Pred_model <- apollo_loadModel("Estimation_results/mxl/prediction/Split_samples/MXL_wtp_Control_Pred")
MXL_wtp_Opt_Not_Pred_model <- apollo_loadModel("Estimation_results/mxl/prediction/Split_samples/MXL_wtp_Opt_Not_Pred")
MXL_wtp_Opt_Pred_model <- apollo_loadModel("Estimation_results/mxl/prediction/Split_samples/MXL_wtp_Opt_Pred")
MXL_wtp_Treated_Not_Pred_model <- apollo_loadModel("Estimation_results/mxl/prediction/Split_samples/MXL_wtp_Treated_Not_Pred")
MXL_wtp_Treated_Pred_model <- apollo_loadModel("Estimation_results/mxl/prediction/Split_samples/MXL_wtp_Treated_Pred")

MXL_wtp_opt_treated <- apollo_loadModel("Estimation_results/mxl/Split_samples/MXL_wtp Vol_Treat")
MXL_wtp_opt_not_treated <- apollo_loadModel("Estimation_results/mxl/Split_samples/MXL_wtp Vol_Not_Treat")

mxl_tr <- apollo_loadModel("Estimation_results/mxl/Split_samples/MXL_wtp Treated A")
mxl_vol_tr <- apollo_loadModel("Estimation_results/mxl/Split_samples/MXL_wtp Vol_Treated A")
mxl_not_tr <- apollo_loadModel("Estimation_results/mxl/Split_samples/MXL_wtp Not_Treated A")

# List of models
models <- list(
  "Treated Not Pred" = MXL_wtp_Treated_Not_Pred_model,
  "Opt Not Pred" = MXL_wtp_Opt_Not_Pred_model,
  "Control Not Pred" = MXL_wtp_Control_Not_Pred_model,
  "Treated Pred" = MXL_wtp_Treated_Pred_model,
  "Opt Pred" = MXL_wtp_Opt_Pred_model,
  "Control Pred" = MXL_wtp_Control_Pred_model,
  "Treated Split Sample" = mxl_tr,
  "Opt Split Sample" = mxl_vol_tr,
  "Control Split Sample" = mxl_not_tr
)

# Initialize an empty list to store results
model_data_list <- list()

# Loop through the models and extract coefficients
for (model_name in names(models)) {
  model <- models[[model_name]]
  
  # Extract mean and standard deviation for 'mu_natural' and 'sig_natural'
  coef <- model$estimate
  se <- model$se
  
  temp_data <- data.frame(
    model = model_name,
    mu = coef["mu_natural"],
    sigma = coef["sig_natural"],
    se_mu = se["mu_natural"],
    se_sigma = se["sig_natural"],
    n = model$nIndivs
  )
  
  # Store in list
  model_data_list[[model_name]] <- temp_data
}

# Combine all data into a single data frame
model_data <- bind_rows(model_data_list)

# Define the pairs of models for comparison
model_pairs <- list(
  c("Treated Split Sample", "Control Split Sample"),
  c("Treated Split Sample", "Opt Split Sample"),
  c("Opt Split Sample", "Control Split Sample"),
  c("Treated Not Pred", "Treated Pred"),
  c("Opt Not Pred", "Opt Pred"),
  c("Control Not Pred" , "Control Pred" ),
  c("Treated Not Pred", "Control Not Pred"),
  c("Opt Not Pred", "Control Not Pred"),
  c("Treated Not Pred", "Opt Not Pred"),
  c("Treated Pred", "Control Pred"),
  c("Opt Pred", "Control Pred"),
  c("Treated Pred", "Opt Pred")
)

# Initialize an empty list to store results
results_list <- list()

# Number of simulations
n_sim <- 1000  # Adjust as needed

# Loop over model pairs
for (pair in model_pairs) {
  model_x <- pair[1]
  model_y <- pair[2]
  
  # Extract relevant data
  x_data <- model_data %>% filter(model == model_x)
  y_data <- model_data %>% filter(model == model_y)
  
  # Run the Lapage function
  result <- lapage(
    x_data$mu, x_data$se_mu, x_data$sigma, x_data$se_sigma, x_data$n,
    y_data$mu, y_data$se_mu, y_data$sigma, y_data$se_sigma, y_data$n,
    model_x, model_y, n_sim
  )
  
  # Store results
  results_list[[paste(model_x, "vs", model_y)]] <- data.frame(
    model_x = model_x,
    model_y = model_y,
    prop_reject = mean(result$p < 0.05) * 100.00
  )
}

# Combine results into a single data frame
final_results <- bind_rows(results_list)

# Print or save results
print(final_results)











# Initialize an empty list to store results
model_data_list <- list()

# Loop through the models and extract coefficients
for (model_name in names(models)) {
  model <- models[[model_name]]
  
  # Extract mean and standard deviation for 'mu_natural' and 'sig_natural'
  coef <- model$estimate
  se <- model$se
  
  temp_data <- data.frame(
    model = model_name,
    mu = coef["mu_walking"],
    sigma = coef["sig_walking"],
    se_mu = se["mu_walking"],
    se_sigma = se["sig_walking"],
    n = model$nIndivs
  )
  
  # Store in list
  model_data_list[[model_name]] <- temp_data
}

# Combine all data into a single data frame
model_data <- bind_rows(model_data_list)

# Define the pairs of models for comparison
model_pairs <- list(
  c("Treated Split Sample", "Control Split Sample"),
  c("Treated Split Sample", "Opt Split Sample"),
  c("Opt Split Sample", "Control Split Sample"),
  c("Treated Not Pred", "Treated Pred"),
  c("Opt Not Pred", "Opt Pred"),
  c("Control Not Pred" , "Control Pred" ),
  c("Treated Not Pred", "Control Not Pred"),
  c("Opt Not Pred", "Control Not Pred"),
  c("Treated Not Pred", "Opt Not Pred"),
  c("Treated Pred", "Control Pred"),
  c("Opt Pred", "Control Pred"),
  c("Treated Pred", "Opt Pred")
)

# Initialize an empty list to store results
results_list <- list()

# Number of simulations
n_sim <- 1000  # Adjust as needed

# Loop over model pairs
for (pair in model_pairs) {
  model_x <- pair[1]
  model_y <- pair[2]
  
  # Extract relevant data
  x_data <- model_data %>% filter(model == model_x)
  y_data <- model_data %>% filter(model == model_y)
  
  # Run the Lapage function
  result <- lapage(
    x_data$mu, x_data$se_mu, x_data$sigma, x_data$se_sigma, x_data$n,
    y_data$mu, y_data$se_mu, y_data$sigma, y_data$se_sigma, y_data$n,
    model_x, model_y, n_sim
  )
  
  # Store results
  results_list[[paste(model_x, "vs", model_y)]] <- data.frame(
    model_x = model_x,
    model_y = model_y,
    prop_reject = mean(result$p < 0.05) * 100.00
  )
}

# Combine results into a single data frame
final_results_wd <- bind_rows(results_list)

# Print or save results
print(final_results_wd)
saveRDS(final_results, "Data/Lapage_Nat.RDS")
saveRDS(final_results_wd, "Data/Lapage_Nat.RDS")
